class UserInteraction:

    def request_message(self, test, text):
        raise Exception("Method UserInteraction.request_message is not implemented")

    def request_yes_no(self, test, text):
        raise Exception("Method UserInteraction.request_yes_no is not implemented")

    def request_option(self, test, text, options):
        raise Exception("Method UserInteraction.request_option is not implemented")
