# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'testing_gui_main_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(912, 609)
        self.central_widget = QtWidgets.QWidget(MainWindow)
        self.central_widget.setObjectName("central_widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.central_widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.central_widget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.verticalLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.test_selected_button = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.test_selected_button.setObjectName("test_selected_button")
        self.horizontalLayout_2.addWidget(self.test_selected_button)
        self.test_failed_button = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.test_failed_button.setObjectName("test_failed_button")
        self.horizontalLayout_2.addWidget(self.test_failed_button)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.test_tree = QtWidgets.QTreeWidget(self.verticalLayoutWidget)
        self.test_tree.setColumnCount(0)
        self.test_tree.setObjectName("test_tree")
        self.test_tree.header().setDefaultSectionSize(150)
        self.test_tree.header().setStretchLastSection(False)
        self.verticalLayout_2.addWidget(self.test_tree)
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.label_3 = QtWidgets.QLabel(self.verticalLayoutWidget_2)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.board_number_spin_box = QtWidgets.QSpinBox(self.verticalLayoutWidget_2)
        self.board_number_spin_box.setMaximumSize(QtCore.QSize(100, 16777215))
        self.board_number_spin_box.setAlignment(QtCore.Qt.AlignCenter)
        self.board_number_spin_box.setMaximum(1000000)
        self.board_number_spin_box.setObjectName("board_number_spin_box")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.board_number_spin_box)
        self.label_2 = QtWidgets.QLabel(self.verticalLayoutWidget_2)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.chip_name_line_edit = QtWidgets.QLineEdit(self.verticalLayoutWidget_2)
        self.chip_name_line_edit.setMouseTracking(False)
        self.chip_name_line_edit.setFocusPolicy(QtCore.Qt.NoFocus)
        self.chip_name_line_edit.setAlignment(QtCore.Qt.AlignCenter)
        self.chip_name_line_edit.setReadOnly(True)
        self.chip_name_line_edit.setObjectName("chip_name_line_edit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.chip_name_line_edit)
        self.verticalLayout_5.addLayout(self.formLayout)
        self.current_test_tab_widget = QtWidgets.QTabWidget(self.verticalLayoutWidget_2)
        self.current_test_tab_widget.setObjectName("current_test_tab_widget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.current_test_description_label = QtWidgets.QLabel(self.tab)
        self.current_test_description_label.setAlignment(QtCore.Qt.AlignCenter)
        self.current_test_description_label.setWordWrap(True)
        self.current_test_description_label.setObjectName("current_test_description_label")
        self.verticalLayout_4.addWidget(self.current_test_description_label)
        self.current_test_tab_widget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.current_test_log_plain_text_edit = QtWidgets.QPlainTextEdit(self.tab_2)
        self.current_test_log_plain_text_edit.setUndoRedoEnabled(False)
        self.current_test_log_plain_text_edit.setReadOnly(True)
        self.current_test_log_plain_text_edit.setObjectName("current_test_log_plain_text_edit")
        self.verticalLayout_3.addWidget(self.current_test_log_plain_text_edit)
        self.current_test_tab_widget.addTab(self.tab_2, "")
        self.verticalLayout_5.addWidget(self.current_test_tab_widget)
        self.verticalLayout.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.central_widget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 912, 20))
        self.menubar.setObjectName("menubar")
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setObjectName("menu")
        self.settings = QtWidgets.QMenu(self.menubar)
        self.settings.setObjectName("settings")
        self.menu_info = QtWidgets.QMenu(self.menubar)
        self.menu_info.setObjectName("menu_info")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.exit_action = QtWidgets.QAction(MainWindow)
        self.exit_action.setObjectName("exit_action")
        self.board_number_auto_inc_action = QtWidgets.QAction(MainWindow)
        self.board_number_auto_inc_action.setCheckable(True)
        self.board_number_auto_inc_action.setObjectName("board_number_auto_inc_action")
        self.actionDebian = QtWidgets.QAction(MainWindow)
        self.actionDebian.setObjectName("actionDebian")
        self.actionRDP_Demo = QtWidgets.QAction(MainWindow)
        self.actionRDP_Demo.setObjectName("actionRDP_Demo")
        self.actionRDP_Demo_2 = QtWidgets.QAction(MainWindow)
        self.actionRDP_Demo_2.setObjectName("actionRDP_Demo_2")
        self.template_2 = QtWidgets.QAction(MainWindow)
        self.template_2.setCheckable(True)
        self.template_2.setChecked(True)
        self.template_2.setObjectName("template_2")
        self.action_save_log = QtWidgets.QAction(MainWindow)
        self.action_save_log.setObjectName("action_save_log")
        self.reset_status_log_action = QtWidgets.QAction(MainWindow)
        self.reset_status_log_action.setObjectName("reset_status_log_action")
        self.about_qt_action = QtWidgets.QAction(MainWindow)
        self.about_qt_action.setObjectName("about_qt_action")
        self.menu.addAction(self.reset_status_log_action)
        self.menu.addSeparator()
        self.menu.addAction(self.exit_action)
        self.settings.addAction(self.board_number_auto_inc_action)
        self.menu_info.addAction(self.about_qt_action)
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.settings.menuAction())
        self.menubar.addAction(self.menu_info.menuAction())

        self.retranslateUi(MainWindow)
        self.current_test_tab_widget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Система запуска тестов"))
        self.test_selected_button.setText(_translate("MainWindow", "Запустить выбранные тесты"))
        self.test_failed_button.setText(_translate("MainWindow", "Запустить упавшие тесты"))
        self.label_3.setText(_translate("MainWindow", "Заводской № платы"))
        self.label_2.setText(_translate("MainWindow", "Чип/платформа"))
        self.current_test_tab_widget.setTabText(self.current_test_tab_widget.indexOf(self.tab), _translate("MainWindow", "Описание теста"))
        self.current_test_tab_widget.setTabText(self.current_test_tab_widget.indexOf(self.tab_2), _translate("MainWindow", "Лог выполнения"))
        self.menu.setTitle(_translate("MainWindow", "Файл"))
        self.settings.setTitle(_translate("MainWindow", "Настройки"))
        self.menu_info.setTitle(_translate("MainWindow", "Справка"))
        self.exit_action.setText(_translate("MainWindow", "Выход"))
        self.board_number_auto_inc_action.setText(_translate("MainWindow", "Автоматически увеличивать зав. № платы"))
        self.actionDebian.setText(_translate("MainWindow", "&Debian"))
        self.actionRDP_Demo.setText(_translate("MainWindow", "&RDP Demo"))
        self.actionRDP_Demo_2.setText(_translate("MainWindow", "RD&P Demo"))
        self.template_2.setText(_translate("MainWindow", "МВ115.&01 (mm7705, процессор-11)"))
        self.action_save_log.setText(_translate("MainWindow", "Сохранить лог"))
        self.reset_status_log_action.setText(_translate("MainWindow", "Сбросить статусы и логи"))
        self.about_qt_action.setText(_translate("MainWindow", "Информация о Qt..."))
